<?php

namespace apexl\Io\modules\display\pages;

use apexl\Io\includes\System;
use apexl\Io\modules\display\components\ImageTile;
use apexl\Io\modules\display\includes\Page;
use apexl\Io\modules\display\regions\ContentRegion;
use apexl\Io\modules\display\regions\ModalContentRegion;
use apexl\Io\modules\display\regions\NoUIContentRegion;
use apexl\Io\modules\display\regions\PageContentRegion;

/**
 * Class blankPage
 * A blank page with no UI, only a content region.
 * @package apexl\Io\modules\display\pages
 */
class modal extends Page
{
    protected function buildRegions()
    {
        $this->globalRegion = (new ModalContentRegion());

    }

    /**
     * @param $component
     * @param null $weight
     */
    public function addToContent($component, $weight = null)
    {
        $this->addToRegion($component, ModalContentRegion::class, $weight);
        return $this;
    }
}
