<?php

namespace apexl\Io\modules\display\components;

use apexl\Io\modules\component\entities\componentEntity;

class ThePageContent extends componentEntity
{
    private static array $wrapperClasses = [
        'h-100'
    ];

    public function __toString()
    {
        $this->addClass(self::$wrapperClasses);
        return parent::__toString();
    }


}
